/*..........................................................................*/
/*                                                                          */
/*      L a s t W a v e   K e r n e l   3 . 0                               */
/*                                                                          */
/*      Copyright (C) 1998-2002 Emmanuel Bacry.                             */
/*      email : lastwave@cmap.polytechnique.fr                              */
/*                                                                          */
/*..........................................................................*/
/*                                                                          */
/*      This program is a free software, you can redistribute it and/or     */
/*      modify it under the terms of the GNU General Public License as      */
/*      published by the Free Software Foundation; either version 2 of the  */
/*      License, or (at your option) any later version                      */
/*                                                                          */
/*      This program is distributed in the hope that it will be useful,     */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of      */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       */
/*      GNU General Public License for more details.                        */
/*                                                                          */
/*      You should have received a copy of the GNU General Public License   */
/*      along with this program (in a file named COPYRIGHT);                */
/*      if not, write to the Free Software Foundation, Inc.,                */
/*      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA             */
/*                                                                          */
/*..........................................................................*/


/****************************************************************************/
/*                                                                          */
/*  misc.c    miscellaneous Functions                                       */
/*                                                                          */
/****************************************************************************/




#include "lastwave.h"
#include <time.h>
#include "xx_system.h"
#include "signals.h"




/*
 * Quick Sort 
 *
 *    Sorts the array of double 'x' which is of size 'n'
 *
 * WARNING : 'x' starts at index 1 (up to 'n' included)
 */

void QuickSort(double *x,int n)
{
  int l,j,ir,i;
  double xx;

  l = ( n>>1)+1;
  ir = n;
  for (;;) {
    if(l>1) xx = x[--l];
    else {
      xx = x[ir];
      x[ir] = x[1];
      if( --ir ==1) { x[1]=xx; return; }
    }
    i=l;
    j=l<<1;
    while( j<= ir) {
      if( j<ir && x[j]< x[j+1]) ++j;
      if(xx < x[j]) {
        x[i] = x[j]; j+=(i=j);
      }
      else j=ir+1;
    }
    x[i] = xx;
  }
}


/*
 * Command to call a unix command
 */
 
void C_SystemUnix(char **argv)
{
  char *cmd;

  cmd = List2Str(argv," ");
  TempStr(cmd);
  
  system("stty iexten"); 
  system("stty ixon"); 
  system("stty ixoff"); 
  system("stty echonl");
  system("stty -noflsh"); 
  system("stty echo");
  system("stty icanon");
  
  system(cmd);

  system("stty -icanon min 1");
  system("stty -echo");
  system("stty -iexten"); 
  system("stty -ixon"); 
  system("stty -ixoff");   
  system("stty -echonl");
  system("stty noflsh"); 
}



/* MyRandom Uniform number */ 

# define MBIG 1000000000
# define MSEED 161803398
# define MZ 0
# define FAC (1.0/MBIG)

static int iff=0;
static long int idum = -1;

void RandInit(long int idum1)
{
  iff = 0;
  idum = idum1;
}

void C_RandInit(char **argv)
{
  int i;
  
  ParseArgv(argv,tINT_,-1,&i,0);
  RandInit(i);
}

LWFLOAT Urand(void)
{
  static int inext,inextp;
  static long ma[56];
  register long mj,mk;
  register int i,ii,k;

  if ( idum <0 || iff == 0){
     iff = 1;
     if (idum < 0) idum = -time(NULL);
     mj = MSEED - ( idum < 0 ? -idum : idum); 
     mj %= MBIG;                         
     ma[55] = mj;
     mk = 1;
     for ( i=1; i<=54; i++){
       ii = ( 21*i) % 55;
       ma[ii] = mk;
       mk = mj - mk;
       if ( mk < MZ ) mk += MBIG;
       mj = ma[ii];
     }
     for ( k=1; k<=4; k++)
       for (i=1; i<=55; i++){
	 ma[i] -= ma[1 + (i+30) % 55];
	 if ( ma[i] < MZ ) ma[i] += MBIG;
       }
     inext = 0;
     inextp = 31;
     idum = 1;
   }
   if( ++inext == 56 )inext=1;
   if( ++inextp == 56 )inextp=1;
   mj = ma[inext] - ma[inextp];
   if( mj < MZ) mj += MBIG;
   ma[inext] = mj;
   return mj*FAC;
}


/* MyRandom Gaussian number */ 

LWFLOAT Grand(LWFLOAT sigma)
{
   static int iset=0;
   static LWFLOAT gset;
   LWFLOAT fac,rsq,v1,v2;

   if (iset == 0) {
    do {
         v1 = 2.0*Urand()-1.0;
         v2 = 2.0*Urand()-1.0;
         rsq=v1*v1+v2*v2;
        }
    while (rsq >= 1.0 || rsq == 0.0);
  fac=sqrt(-2.0*log(rsq)/rsq);
  gset = v1*fac;
  iset=1;
  return(v2*fac*sigma);
 }
 else {
  iset = 0;
  return(sigma*gset);
 }
}



LWFLOAT Prand(LWFLOAT xm)
{
  static LWFLOAT sq,alxm,g,oldm=(-1.0);
  LWFLOAT em,t,y;
  long int intem,idum=1;
  
  if (xm< 12.0) {
    if (xm != oldm){
      oldm = xm;
      g = exp(-xm);
    }
    em = -1;
    t=1.0;
    do {
      ++em;
      t *= Urand();
    } while(t>g);
  } 
  else {
    if (xm != oldm){
      oldm = xm;
      sq = sqrt(2.0*xm);
      alxm=log(xm);
      g=xm*alxm-gammln(xm+1.0);
    }
    do {
      do {
        y = tan(3.141592654*Urand());
        em = sq*y+xm;
      } while(em <0.0);
      intem = em;
      em = intem;
      t = 0.9*(1.0+y*y)*exp(em*alxm-gammln(em+1.0)-g);
    }while(Urand() > t);
  }
  return(em);
}

void C_PRand(char **argv) 
{
  int size;
  LWFLOAT mean;
  SIGNAL sig;
  int i;
  
  argv = ParseArgv(argv,tINT,&size,tFLOAT,&mean,0);
  
  sig = NewSignal();
  SizeSignal(sig,size,YSIG);
  for (i = 0;i<size;i++) sig->Y[i] = Prand(mean); 
  
  SetResultValue((VALUE) sig);
  DeleteSignal(sig);
}
  

/**************************************************
 *
 * Functions that deal with time
 *
 **************************************************/

/*
 * Get the number of seconds since 1 January 1970 00h00
 */
LWFLOAT MyTime(void) 
{
  long sec,microsec;
  XXGetTime(&sec,&microsec);
  
  return(sec+microsec*1.e-6);
}


/*
 * Wait function
 */
void Wait(LWFLOAT delay) 
{
  LWFLOAT t;
  
  if (delay <= 0) return;

  t = MyTime()+delay;
  
  while(MyTime() < t) {
//    ProcessNextEvent(YES,t-MyTime());
  }  
}

void C_Wait(char **argv)
{
  LWFLOAT delay;
  
  argv = ParseArgv(argv,tFLOAT,&delay,0);

  Wait(delay);
}


// Get current date and time
void GetCurrentDate(int *d,int *m, int *y, int *h, int *mn, int *s, int flagGMT)
{
  time_t now = time(NULL);
  struct tm *tm;
  
  if (flagGMT) tm  = gmtime(&now);
  else tm = localtime(&now);

  *y = tm->tm_year+1900;  
  *d = tm->tm_mday;  
  *m = tm->tm_mon+1;  
  *h = tm->tm_hour;  
  *mn = tm->tm_min;  
  *s = tm->tm_sec;  
}


// Corresponding command
void C_Time(char **argv)
{
  extern int YYYYMMDD(int yy, int mm, int dd);
  LISTV lv;
  int d,m,y,h,mn,s;
  char *action,opt;
  LWFLOAT ymd,hms,sec;
  char flagDate,flagListv,flagGMT;
  
  argv = ParseArgv(argv,tWORD,&action,-1);
    
  // time sec
  if (!strcmp(action,"sec")) {
    NoMoreArgs(argv);
    sec = MyTime();
    SetResultFloat(sec);
    return;
  }

  // time 2sec
  if (!strcmp(action,"2sec")) {
    argv = ParseArgv(argv,tLISTV,&lv,0);
    if (lv->length != 2) ErrorUsage();
    NoMoreArgs(argv);
    ymd = GetListvNthFloat(lv,0);
    if (ymd != (int) ymd) ErrorUsage();
    hms = (int) GetListvNthFloat(lv,1);
    if (hms != (int) hms) ErrorUsage();
    sec = (Date2Index((int) ymd)-Date2Index(19700101))*24*3600;
    h = (int) (hms/10000);
    hms -= h*10000;
    mn = (int) (hms/100);
    hms -= mn*100;
    sec += hms + mn*60+h*3600; 
    SetResultFloat(sec);
    return;
  }

  // time sec2
  if (!strcmp(action,"sec2")) {
    argv = ParseArgv(argv,tFLOAT,&sec,0);
    if (sec != (int) sec) ErrorUsage();
    ymd = Index2Date((((int) sec) / (3600*24)) + Date2Index(19700101));
    sec -= (((int) sec) / (3600*24))*3600*24;
    h = ((int) sec)/3600;
    sec -= h*3600;
    mn = ((int) sec)/60;
    sec -= mn*60;
    s = ((int) sec);
    hms = h*10000+mn*100+s;
    lv = TNewListv();
    AppendFloat2Listv(lv,ymd);
    AppendFloat2Listv(lv,hms);
    SetResultValue((VALUE) lv);
    return;
  }

  // time s2hms
  if (!strcmp(action,"s2hms")) {
    argv = ParseArgv(argv,tFLOAT,&sec,0);
    h = ((int) sec)/3600;
    sec -= h*3600;
    mn = ((int) sec)/60;
    sec -= mn*60;
    lv = TNewListv();
    AppendFloat2Listv(lv,h);
    AppendFloat2Listv(lv,mn);
    AppendFloat2Listv(lv,sec);
    SetResultValue((VALUE) lv);
    return;
  }

  // time date
  if (!strcmp(action,"current")) {
    flagDate = NO;
    flagListv = NO;
    flagGMT = NO;
    while(opt = ParseOption(&argv)) { 
      switch(opt) {
      case 'd':
        flagDate = YES; 
        break;
      case 'g':
        flagGMT = YES; 
        break;
      case 'l': 
        flagListv = YES;
        break;
      default : ErrorOption(opt);
      }
    }  
    NoMoreArgs(argv);  

    GetCurrentDate(&d,&m,&y,&h,&mn,&s,flagGMT);

    if (flagDate) {
      if (flagListv) {    
        lv = TNewListv();
        AppendInt2Listv(lv,d);
        AppendInt2Listv(lv,m);
        AppendInt2Listv(lv,y);
        AppendInt2Listv(lv,h);
        AppendInt2Listv(lv,mn);
        AppendInt2Listv(lv,s);
        NoMoreArgs(argv);
        SetResultValue(lv);
        return;
      }
      else {
        lv = TNewListv();
        AppendFloat2Listv(lv,YYYYMMDD(y,m,d));
        AppendFloat2Listv(lv,h*10000+mn*100+s);
        SetResultValue(lv);
        return;
      }
    }
    else {
      if (flagListv) {    
        lv = TNewListv();
        AppendInt2Listv(lv,h);
        AppendInt2Listv(lv,mn);
        AppendInt2Listv(lv,s);
        NoMoreArgs(argv);
        SetResultValue(lv);
        return;
      }
      else {
        SetResultFloat(h*10000+mn*100+s);
        return;
      }
    }
    return;
  }

  Errorf("Unknown action '%s'",action);
}  

/* Test for bissextile years */
static char IsBissextile(int n)
{
  if ((n % 400 == 0) || (n % 4 == 0) && (!(n % 100 == 0))) return(YES);
  else return(NO);
}

// Returns the year from format YYYYMMDD
int YYYY(int yymmdd)
{
    int dd,mm;
    dd = yymmdd % 100;
    mm = (yymmdd-dd) % 10000;
    return ((yymmdd-mm-dd)/10000);
}

// Returns the month from format YYYYMMDD
int MM(int yymmdd)
{
    int dd;
    dd = yymmdd % 100;
    return (((yymmdd-dd) % 10000)/100);
}

// Returns the day from format YYYYMMDD
int DD(int yymmdd)
{
    return((yymmdd % 100));
}

// Returns the date index (i.e., number of days since 1/01/1900)
#define YEAR_ORIGIN 1900
int Date2Index(int yymmdd)
{
    int yy = YYYY(yymmdd);
    int mm = MM(yymmdd);
    int dd = DD(yymmdd);
    int nb,i;
    char flagB;

    if (yy < YEAR_ORIGIN) 
	Errorf("Date2Index: The year must be after (or equal to) YEAR_ORIGIN");

    nb=0;
    for(i=YEAR_ORIGIN;i<yy;++i) {
	  if (IsBissextile(i)) nb += 366;
	  else nb += 365;
      }
      flagB = IsBissextile(yy);
      for(i=1;i<mm;++i)
	  switch(i)
	  {
	      case 1:
	      case 3:
	      case 5:
	      case 7:
	      case 8:
	      case 10: 
		  nb += 31;
		  break;
	      case 2: 
		  if (flagB) nb+= 29;
		  else nb += 28;
		  break;
	      case 4:
	      case 6:
	      case 9:
	      case 11: 
		  nb += 30;
		  break;
	  }
      nb += dd;
      return(nb);
}

// Returns the date under format YYYYMMDD
int YYYYMMDD(int yy, int mm, int dd)
{
    return(yy*10000+mm*100+dd);
}

// Inverse function of the previous one
int Index2Date(LWFLOAT ndays1)
{
    int ndays = (int) ndays1;
    int yy=YEAR_ORIGIN,mm=1,dd=1;
    if (ndays != ndays1) Errorf("Index2Date() : index is a float (%g)\n",ndays1);
    

    if (ndays < 0) Errorf("Index2Date: The year must be after TimeOrigin");

    while (Date2Index(YYYYMMDD(yy,1,1)) <= ndays) yy++;
    yy -= 1;

    while ((Date2Index(YYYYMMDD(yy,mm,1)) <= ndays) && (mm <= 12)) mm++;
    if (mm > 1) mm -= 1;
    else mm = 1;

    while ((Date2Index(YYYYMMDD(yy,mm,dd)) <= ndays) && (dd <= 31)) dd++;
    if (dd > 1) dd -= 1;
    else dd = 1;

    return(YYYYMMDD(yy,mm,dd));
}


// Get the week day number from index
#define DATEREF 20040326
#define DAYREF 5
int Index2Day(LWFLOAT ndays1)
{
  static int base = -1;

  int ndays = (int) ndays1;
  if (ndays != ndays1) Errorf("Index2Day() : index is a float (%g)\n",ndays1);

  if (base == -1) base = ((DAYREF-1) - Date2Index(DATEREF)) % 7;

  return (((ndays + base) %7) + 1);
}

// Check if a date is a valid YYYYMMDD date
char IsValidYYYYMMDD(int yyyymmdd)
{
    int dd,mm,yyyy;
    dd = yyyymmdd % 100;
    mm = (yyyymmdd-dd) % 10000;
    yyyy =  (yyyymmdd-mm-dd)/10000;
    mm /= 100;
    
    if (mm<1 || mm>12) return(NO);
    if (dd<1) return(NO);
    switch (mm) {
      case 1 : if (dd>31) return(NO); break;
      case 2 : 
        if (dd>29) return(NO);
        if (!((yyyy % 400 == 0) || (yyyy % 4 == 0) && (!(yyyy % 100 == 0))) && dd==29) return(NO); 
        break;
      case 3 : if (dd>31) return(NO); break;
      case 4 : if (dd>30) return(NO); break;
      case 5 : if (dd>31) return(NO); break;
      case 6 : if (dd>30) return(NO); break;
      case 7 : if (dd>31) return(NO); break;
      case 8 : if (dd>31) return(NO); break;
      case 9 : if (dd>30) return(NO); break;
      case 10 : if (dd>31) return(NO); break;
      case 11 : if (dd>30) return(NO); break;
      case 12 : if (dd>31) return(NO); break;
    }
    
    return(YES);
}


// Corresponding command
void C_Date(char **argv)
{
  LISTV lv;
  int d,m,y,h,mn,s,date,index;
  char *action,opt;
  int flagGMT,flagListv;
  
  argv = ParseArgv(argv,tWORD,&action,-1);
  
  // date current
  if (!strcmp(action,"current")) {
    flagGMT = NO;
    flagListv = NO;
    while(opt = ParseOption(&argv)) { 
      switch(opt) {
      case 'l': 
        flagListv = YES;
        break;
      case 'g': 
        flagGMT = YES;
        break;
      default : ErrorOption(opt);
      }
    }  
    NoMoreArgs(argv);  
    GetCurrentDate(&d,&m,&y,&h,&mn,&s,flagGMT);
    if (flagListv) {
      lv = TNewListv();
      AppendInt2Listv(lv,d);
      AppendInt2Listv(lv,m);
      AppendInt2Listv(lv,y);
      NoMoreArgs(argv);
      SetResultValue(lv);
      return;
    }      
    SetResultFloat(y*10000+m*100+d);
    return;
  }

  // date 2index
  if (!strcmp(action,"2index")) {
    argv = ParseArgv(argv,tINT,&date,0);
    if (!IsValidYYYYMMDD(date)) Errorf("Invalid date yyyymmdd '%d'",date);
    SetResultInt(Date2Index(date));
    return;
  }

  // date index2
  if (!strcmp(action,"index2")) {
    argv = ParseArgv(argv,tINT,&index,0);
    SetResultInt(Index2Date(index));
    return;
  }

  // date day
  if (!strcmp(action,"day")) {
    argv = ParseArgv(argv,tINT,&index,0);
    SetResultInt(Index2Day(index));
    return;
  }
  
  Errorf("Unknown action '%s'",action);
}  



/**************************************************
 *
 * Functions that deal with little/big endian
 *
 **************************************************/

/* Test whether we are running on big endian or little endian and set the corresponding variable */
char cpuBinaryMode;
void  InitCPUBinaryMode(void) 
{ 
  short unsigned int i;
  char *c;
  
  i = 1;
  c = (char *) (&i);
  if (*c == 0) cpuBinaryMode = BinaryBigEndian;
  else cpuBinaryMode = BinaryLittleEndian;
} 
  

/* 
 * Function to convert a Big (resp. Little) Endian array of values to a Little (resp. Big) array of values 
 *
 *    array   : is the array of values
 *    n       : the number of values
 *    sizeval : the number of bytes of each value
 */
void BigLittleValues(void *array, int n, size_t sizeval)
{
  size_t i;
  int j;
  unsigned char c;
  unsigned char *pvar;
  
  for (j=0;j<n*sizeval;j+=sizeval) {
    
    pvar = ((unsigned char *) array) + j;
  
    for(i=0;i<sizeval/2;i++) {
      c = *(pvar+i);
      *(pvar+i) = *(pvar+sizeval-1-i);
      *(pvar+sizeval-1-i) = c;
    }
  }
}


