// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"


// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif


#include "wx/image.h"

#include "wx/choicdlg.h"
#include "wx/fontdlg.h"
#include "wx/fontenum.h"
#include "wx/fontmap.h"
#include "wx/encconv.h"
#include "wx/splitter.h"
#include "wx/textfile.h"


#ifdef __WXMAC__
#undef wxFontDialog
#include "wx/mac/fontdlg.h"
#endif


extern "C" {
  
#include "lastwave.h"
  
}


#include "terminalCtrl.h"
#include "wxmain.h"
#include "wxgraphics.h"



#define FRAME void *

#define FONTID int *

// wxFontEncoding greekEncoding = wxFONTENCODING_ISO8859_7;
wxFontEncoding greekEncoding = wxFONTENCODING_DEFAULT;

// Managing 1/2 button mouse
static long leftButton = LeftButton; 
static long rightButton = RightButton; 
static long middleButton = MiddleButton; 
static int nbOfButtons = 3;

long XXMouseButton(wxMouseEvent& event)
{
  long mask;
  
  if (nbOfButtons == 1) {
    mask = LeftButton;
    if (event.MetaDown()) mask += ModMeta;
    if (event.AltDown()) mask += ModOpt;
  }
  else if (nbOfButtons == 2) {
    if (event.LeftDown()) mask = LeftButton;
    else if (event.RightDown()) mask = RightButton;
    else if (event.MiddleDown()) mask = MiddleButton;
    if (event.MetaDown()) mask += ModMeta;
    if (event.AltDown()) mask += ModOpt;
  }
  else {
    if (event.LeftDown()) mask = LeftButton;
    else if (event.RightDown()) mask = RightButton;
    else if (event.MiddleDown()) mask = MiddleButton;
  }
  
  
  if (leftButton == mask) return(LeftButton);
  else if (middleButton == mask) return(MiddleButton);
  else if (rightButton == mask) return(RightButton);
  else return(NoButton);
}



static wxPalette *thePalette = NULL;
static wxPen thePen;
static wxBrush theBrush;
static int thePenMode;
static unsigned char theR, theG, theB;
static int theClipX,theClipY,theClipW,theClipH;
static wxFont theFont;


//
// Sometimes we want to block events, i.e., to forget about any event that takes place.
// In any case we never skip the terminal keyDown events. We buffer them in order to reproduce them later.
//
// There are 2 reasons for blocking events :
//
// 1) When a non terminal event is processed, we do not want any other event to interfere
//    In that case all events must be blocked (except the keydown terminal ones of course)
//    --> Macro BlockEvent
// 2) When something is printed on the terminal we make a Yield call. We do not want major things to happen during this call.
//    We want to block all events except maybe repaint events and mouseUp events
//    --> Macro BlockEventSpecial
//

static bool eventBlocked = false;
static bool eventBlockedByTerminal = false;
bool IsEventBlocked() 
{
  return(eventBlocked);
}
bool IsEventBlockedByTerminal()
{
  return(eventBlockedByTerminal);
}
void BlockEvent()
{
  eventBlocked = true;
  eventBlockedByTerminal = false;
}
void BlockEventByTerminal()
{
  eventBlocked = true;
  eventBlockedByTerminal = true;
}
void UnBlockEvent()
{
  eventBlocked = false;
  eventBlockedByTerminal = false;
}

class Frame;

class Canvas : public wxWindow
// public wxScrolledWindow
{
  public :
  
  Canvas(Frame *f);
    ~Canvas();

    Frame *frame;
    wxDC *dc;
    
	// Canvas Border
	int hBorder, wBorder;
	
	int mouseButton;
	
  bool flagResizeEvent;
    
    void OnPaint(wxPaintEvent& event);
    void OnMouseDown(wxMouseEvent& event);
    void OnMouseUp(wxMouseEvent& event);
    void OnMouseEnterLeave(wxMouseEvent& event);
    void OnMouseMotion(wxMouseEvent& event);
    void OnResize(wxSizeEvent& event);

    void KeyDown(wxKeyEvent& event);
    void KeyChar(wxKeyEvent& event);

    DECLARE_EVENT_TABLE();
};

Canvas::~Canvas()
{
}

BEGIN_EVENT_TABLE(Canvas, /*wxScrolledWindow*/ wxWindow)
  EVT_PAINT(Canvas::OnPaint)
  EVT_LEFT_UP(Canvas::OnMouseUp)
  EVT_MIDDLE_UP(Canvas::OnMouseUp)
  EVT_RIGHT_UP(Canvas::OnMouseUp)
  EVT_LEFT_DOWN(Canvas::OnMouseDown)
  EVT_MIDDLE_DOWN(Canvas::OnMouseDown)
  EVT_RIGHT_DOWN(Canvas::OnMouseDown)
  EVT_ENTER_WINDOW(Canvas::OnMouseEnterLeave)
  EVT_LEAVE_WINDOW(Canvas::OnMouseEnterLeave) 
  EVT_MOTION(Canvas::OnMouseMotion)
  EVT_SIZE(Canvas::OnResize)
  EVT_KEY_DOWN(Canvas::KeyDown)
  EVT_CHAR(Canvas::KeyChar) 
END_EVENT_TABLE()




class Frame : public wxFrame 
{
  public :
  
    Frame(wxString title,int x, int y, int w,int h);
    ~Frame();

    // Frame border
    int hBorder,wBorder;

    // Get Total border
	inline int totalWBorder() {return(wBorder+canvas->wBorder);};
	inline int totalHBorder() {return(hBorder+canvas->hBorder);};
	
    Canvas *canvas;
        
    void Focus(wxFocusEvent& event);

    void OnClose(wxCloseEvent& event);
    void OnMove(wxMoveEvent& event);
    void OnActivate(wxActivateEvent& event);
    
    DECLARE_EVENT_TABLE();
};




BEGIN_EVENT_TABLE(Frame, wxFrame)
  EVT_CLOSE(Frame::OnClose)
  EVT_MOVE(Frame::OnMove)
EVT_ACTIVATE(Frame::OnActivate)
END_EVENT_TABLE()


void Frame::OnClose(wxCloseEvent& event)
{
  BLOCKEVENT;
  
  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this);
  if (ev.object == NULL) Destroy();
  else {
    ev.type = Del;
    ProcessNextEvent(&ev);
  }
  
//  Printf("Close Frame\n");  
  UNBLOCKEVENT;
}

void Frame::OnMove(wxMoveEvent& event)
{
  struct event ev;

	ev.object = (GOBJECT) Frame2Window(this);
  if (ev.object == NULL) {
    return;
  }
  
  BLOCKEVENT;
  wxPoint pos = GetPosition();
  ev.type = Resize;
  
  ev.i = pos.x;
  ev.j = pos.y;
  canvas->GetClientSize(&(ev.m),&(ev.n));
  
  ProcessNextEvent(&ev);
  // Printf("Move Frame %d %d %d %d\n",ev.i,ev.j,ev.m,ev.n);  
  
  UNBLOCKEVENT;
}

void Frame::OnActivate(wxActivateEvent& event)
{
  if (event.GetActive()) canvas->SetFocus();
}


static bool GetLWKeyEvent(wxWindow *window, wxKeyEvent& event, unsigned long key, bool flagShift, bool flagAlt, bool flagCtrl, struct event &ev)
{
  if (window != myApp->terminal->textCtrl) {
    Canvas *c = (Canvas *) window;
    ev.object = (GOBJECT) Frame2Window(c->frame);
    if (ev.object == NULL) return(false);
  }
  else ev.object = NULL;

  ev.type = KeyDown1;
  ev.key = key;

  if (flagCtrl) ev.key += ModCtrl;
  if (flagShift) ev.key += ModShift;
  if (flagAlt) ev.key += ModOpt;
  
  wxPoint p = ::wxGetMousePosition();
  p = window->ScreenToClient(p);
  
  ev.i = event.GetX();  
  ev.j = event.GetY();  

  ev.i = p.x;  
  ev.j = p.y;  
  return(true);
}

bool GetKeyCharLWEvent(wxWindow *w, wxKeyEvent& event, struct event &ev)
{
  // Menu management
  switch(wxGetOsVersion()) {
      __CASEMAC : 
      if (event.CmdDown()) {
        event.Skip();
        return(false);
      }
      break;
      
      __CASEWINDOWS : __CASEUNIX :
      if (!event.AltDown() && event.ControlDown() && !event.MetaDown()) {
        event.Skip();
        return(false);
      }
      break;
    default :
      break;
  }

  // THIS IS HIGH HACKING ....that I do not really master :-)
  if (event.GetKeyCode() < ' ') {
//    event.Skip();
    return(false);
  }
 
  return(GetLWKeyEvent(w,event,event.GetKeyCode(),false,false,false,ev));
}

void Canvas::KeyChar(wxKeyEvent& event) 
{
  BLOCKEVENT;
  struct event ev;
  
  if (GetKeyCharLWEvent(this,event,ev)) ProcessNextEvent(&ev);
  UNBLOCKEVENT;
}


// 
// Dealing with key down
// This routine is called on KeyDown event. 
//
// If the key must be processed using a KeyChar then the event should be skipped and return(false)
// If the key should not be taken into account return(false)
// Otherwise the event 'ev' should be filled and return(true)
//
bool GetKeyDownLWEvent(wxWindow *w, wxKeyEvent& event, struct event &ev) 
{
  // Menu management
  switch(wxGetOsVersion()) {
      __CASEMAC :
      if (event.CmdDown()) {
        event.Skip();
        return(false);
      }
      break;
      __CASEWINDOWS : __CASEUNIX :
      if (!event.AltDown() && event.ControlDown() && !event.MetaDown() && event.GetKeyCode() != 'D') {
        event.Skip();
        return(false);
      }
      break;
    default : 
      break;
  }
  
  // the wxCode of the key
  long code = event.GetKeyCode();
  
  // The LastWave key code
  unsigned long key = 0;
  
  // The name of the key if not a character
  char *t = NULL;

  //
  // The different codes
  // 
  //     -> if return then not taken care
  //     -> if break --> taken care
  //
  
  switch(code) {
    
  case WXK_BACK : t = "delete"; key = DeleteKC; break;
  case WXK_RETURN :  t = "return"; key = NewlineKC; break;
  case WXK_ESCAPE : t = "esc"; key = EscapeKC; break; 
  case WXK_TAB : t = "tab"; key = TabKC; break; 
  case WXK_SPACE : break;
  case WXK_DELETE : t = "delete"; key = DeleteKC; break;

  case WXK_SHIFT : event.Skip(); return(false);
  case WXK_ALT : event.Skip(); return(false);
  case WXK_COMMAND : event.Skip(); return(false);
  case WXK_CONTROL : event.Skip(); return(false); 
  case WXK_CAPITAL : event.Skip(); return(false);

  case WXK_HOME : t = "home"; key = HomeKC; break;
  case WXK_LEFT : t = "left"; key = LeftKC; break;
  case WXK_UP : t = "up"; key = UpKC; break;
  case WXK_RIGHT : t = "right"; key = RightKC; break;
  case WXK_DOWN : t = "down"; key = DownKC; break;

  case WXK_CLEAR : t = "clear"; key = ClearKC; break;
  case WXK_END : t = "end"; key = EndKC; break;

  case WXK_F1 : t = "f1"; key = F1KC; break;
  case WXK_F2 : t = "f2"; key = F2KC; break;
  case WXK_F3 : t = "f3"; key = F3KC; break;
  case WXK_F4 : t = "f4"; key = F4KC; break;
  case WXK_F5 : t = "f5"; key = F5KC; break;
  case WXK_F6 : t = "f6"; key = F6KC; break;
  case WXK_F7 : t = "f7"; key = F7KC; break;
  case WXK_F8 : t = "f8"; key = F8KC; break;
  case WXK_F9 : t = "f9"; key = F9KC; break;
  case WXK_F10 : t = "f10"; key = F10KC; break;
  case WXK_F11 : t = "f11"; key = F11KC; break;
  case WXK_F12 : t = "f12"; key = F12KC; break;
  case WXK_F13 : t = "f13"; key = F13KC; break;
  case WXK_F14 : t = "f14"; key = F14KC; break;
  case WXK_F15 : t = "f15"; key = F15KC; break;

    
  case WXK_START : t = "start"; return(false);
  case WXK_LBUTTON : t = "lb";  return(false);
  case WXK_RBUTTON : t = "rb";  return(false);
  case WXK_CANCEL : t = "cancel"; return(false);
  case WXK_MBUTTON : t = "mb"; break;
  case WXK_MENU : t = "menu"; return(false);
  case WXK_PAUSE : t = "pause"; return(false);
  case WXK_PRIOR : t = "prior"; return(false);
  case WXK_NEXT : t = "next"; return(false);
  case WXK_SELECT : t = "select"; return(false);
  case WXK_PRINT : t = "print"; return(false);
  case WXK_EXECUTE : t = "exec"; return(false);
  case WXK_SNAPSHOT : t = "snap"; return(false);
  case WXK_INSERT : t = "insert"; return(false);
  case WXK_HELP : t = "help"; return(false);
  case WXK_NUMPAD0 : return(false);
  case WXK_NUMPAD1 : return(false);
  case WXK_NUMPAD2 : return(false);
  case WXK_NUMPAD3 : return(false);
  case WXK_NUMPAD4 : return(false);
  case WXK_NUMPAD5 : return(false);
  case WXK_NUMPAD6 : return(false);
  case WXK_NUMPAD7 : return(false);
  case WXK_NUMPAD8 : return(false);
  case WXK_NUMPAD9 : return(false);
  case WXK_MULTIPLY : return(false);
  case WXK_ADD : return(false);
  case WXK_SEPARATOR : return(false);
  case WXK_SUBTRACT : return(false);
  case WXK_DECIMAL : return(false);
  case WXK_DIVIDE : return(false);
  case WXK_F16 : t = "f16"; return(false);
  case WXK_F17 : t = "f17"; return(false);
  case WXK_F18 : t = "f18"; return(false);
  case WXK_F19 : t = "f19"; return(false);
  case WXK_F20 : t = "f20"; return(false);
  case WXK_F21 : t = "f21"; return(false);
  case WXK_F22 : t = "f22"; return(false);
  case WXK_F23 : t = "f23"; return(false);
  case WXK_F24 : t = "f24"; return(false);
  case WXK_NUMLOCK : return(false);
  case WXK_SCROLL  : t = "scroll"; return(false);
 // case WXK_PAGEUP : t = "pageup"; return(false);
 // case WXK_PAGEDOWN : t = "pagedown"; return(false);

  case WXK_NUMPAD_SPACE : return(false);
  case WXK_NUMPAD_TAB  : return(false);
  case WXK_NUMPAD_ENTER  : return(false);
  case WXK_NUMPAD_F1 : t = "nf1"; return(false);
  case WXK_NUMPAD_F2 : t = "nf2"; return(false);
  case WXK_NUMPAD_F3 : t = "nf3"; return(false);
  case WXK_NUMPAD_F4 : t = "nf4"; return(false);
  case WXK_NUMPAD_HOME : t = "nhome"; return(false);
  case WXK_NUMPAD_LEFT : t = "nleft"; return(false);
  case WXK_NUMPAD_UP : t = "nup"; return(false);
  case WXK_NUMPAD_RIGHT : t = "nright"; return(false);
  case WXK_NUMPAD_DOWN : t = "ndown"; return(false);
  case WXK_NUMPAD_PRIOR : t = "nprior"; return(false);
//  case WXK_NUMPAD_PAGEUP : t = "npageup"; return(false);
//  case WXK_NUMPAD_NEXT : t = "nnext"; return(false);
  case WXK_NUMPAD_PAGEDOWN : t = "npagedown"; return(false);
  case WXK_NUMPAD_END : t = "nend"; return(false);
  case WXK_NUMPAD_BEGIN : t = "nbeg"; return(false);
  case WXK_NUMPAD_INSERT : t = "ninsert"; return(false);
  case WXK_NUMPAD_DELETE : t = "ndelete"; return(false);
  case WXK_NUMPAD_EQUAL : return(false);
  case WXK_NUMPAD_MULTIPLY : return(false);
  case WXK_NUMPAD_ADD : return(false);
  case WXK_NUMPAD_SEPARATOR : t = "nsep"; return(false);
  case WXK_NUMPAD_SUBTRACT : return(false);
  case WXK_NUMPAD_DECIMAL : return(false);
  case WXK_NUMPAD_DIVIDE : return(false);
    
  case WXK_WINDOWS_LEFT : t = "wleft"; return(false);
  case WXK_WINDOWS_RIGHT : t = "wrigt"; return(false);
  case WXK_WINDOWS_MENU : t = "wmenu"; return(false);
  
  default : break;
  
  };
    
  // Case the key is printable (i.e., it has no name)
  if (t == NULL) {
  
    switch(wxGetOsVersion()) {
      // On a MAC : 
    // If option key with no control and regular characters --> Char event should deal with it 
        __CASEMAC :
        if (event.AltDown() && !event.ControlDown() && !event.CmdDown() && code >= ' ' && code <= '~') {
          event.Skip();
          return(false);
        }
        break;
        // On WINDOWS : 
        // If alt+Ctrl keys with no meta and regular characters --> Char event should deal with it 
        __CASEWINDOWS : __CASEUNIX :
        if (event.AltDown() && event.ControlDown() && !event.MetaDown() && code >= ' ' && code <= '~') {
      event.Skip();
      return(false);
    }
        break;
      default : 
        break;
    }        
    
    // The next paragraph deal with modifiers 
    if (event.ControlDown()) {  // Case of a control key
      if (code >= 'A' && code <= 'Z') {
        if (!event.ShiftDown()) code += -'A'+'a';
        if (code == 'd' && !event.AltDown()) return(GetLWKeyEvent(w,event,EofKC,false,false,false,ev));
        else return(GetLWKeyEvent(w,event,code,false,event.AltDown(),event.ControlDown(),ev));
      }
      else if (code == WXK_SPACE) {
        return(GetLWKeyEvent(w,event,' ', event.ShiftDown(),event.AltDown(),event.ControlDown(),ev));
      }
	  else if (code >= ' ' && code <= '~') {
      return(GetLWKeyEvent(w,event,code,false,event.AltDown(),event.ControlDown(),ev));
	  }
	  else if (key != 0) {
      return(GetLWKeyEvent(w,event,key,event.ShiftDown(),event.AltDown(),event.ControlDown(),ev));
    }
    }
    
    // Otherwise --> regular characters
    else event.Skip();
  }
  
  // Case there is a name and the key has been assigned
  else if (key != 0) {
    return(GetLWKeyEvent(w,event,key,event.ShiftDown(),event.AltDown(),event.ControlDown(),ev));
  }
  
  return(false);
}

void Canvas::KeyDown(wxKeyEvent& event) 
{
  BLOCKEVENT;
  struct event ev;
  
  if (GetKeyDownLWEvent(this,event,ev)) ProcessNextEvent(&ev);
  UNBLOCKEVENT;
}


// The resize event will be taken care by the Paint event
void Canvas::OnResize(wxSizeEvent& event)
{
  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  BLOCKEVENT;

  Refresh(false);
  flagResizeEvent =true;
  
//    Printf("Resize\n");
  UNBLOCKEVENT;
}

void Canvas::OnMouseEnterLeave(wxMouseEvent& event)
{
  //  event.Skip();
  
  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  BLOCKEVENT;

  if (event.Leaving()) ev.type = Leave;
  else if (event.Entering()) ev.type = Enter;

  wxClientDC dc1(this);
  dc = &dc1;

  ProcessNextEvent(&ev);
//  Printf("Enter/Leave %ld\n",ev.type);
  
  dc = NULL;
  UNBLOCKEVENT;
}

void Canvas::OnMouseMotion(wxMouseEvent& event)
{
  //  event.Skip();
  
  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  BLOCKEVENT;

  ev.type = MouseMotion;
  
  ev.key = 0;

  ev.button = mouseButton;
  
  ev.i = event.GetX();  
  ev.j = event.GetY();  
  
  wxClientDC dc1(this);
  dc = &dc1;

//  Printf("Motion %ld %d %d \n",ev.button,ev.i,ev.j);
  ProcessNextEvent(&ev);
  
  dc = NULL;

  UNBLOCKEVENT;
}

void Canvas::OnMouseUp(wxMouseEvent& event)
{
  if (HasCapture()) ReleaseMouse();
  else return;

  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  BLOCKEVENT;

  ev.type = ButtonUp;
  ev.button = mouseButton;

  ev.key = 0;
  
  ev.i = event.GetX();  
  ev.j = event.GetY();  
  
  wxClientDC dc1(this);
  dc = &dc1;

  ProcessNextEvent(&ev);
//  Printf("UnClick %d %d \n",ev.i,ev.j);
  
  mouseButton = NoButton;
  
  dc = NULL;
  UNBLOCKEVENT;
}

void Canvas::OnMouseDown(wxMouseEvent& event)
{
//  event.Skip();
  struct event ev;

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  ev.button = XXMouseButton(event);
  if (ev.button == NoButton) return;

  BLOCKEVENT;

  ev.type = ButtonDown;
  
  unsigned long m = 0;
  
  if (event.ShiftDown()) m += ModShift;
  if (event.ControlDown()) m += ModCtrl;
  ev.button += m;
  
  ev.key = 0;
  
  ev.i = event.GetX();  
  ev.j = event.GetY();  
  
  wxClientDC dc1(this);
  dc = &dc1;

  CaptureMouse();
  mouseButton = ev.button;
  
  ProcessNextEvent(&ev);
//  Printf("Click %ld %ld %d %d \n",ev.i,ev.j);
  
  dc = NULL;
  UNBLOCKEVENT;

}


void Canvas::OnPaint(wxPaintEvent& event)
{
  int i = 0;
  struct event ev;

  //  event.Skip();

  ev.object = (GOBJECT) Frame2Window(this->frame);
  if (ev.object == NULL) return;

  if (IsEventBlocked() && !IsEventBlockedByTerminal()) return;
  
  wxPaintDC dc1(this);
  dc = &dc1;

  if (flagResizeEvent) {
    flagResizeEvent = false;
    ev.type = Resize;
  
    frame->GetPosition(&(ev.i),&(ev.j));
	GetClientSize(&(ev.m),&(ev.n));
//    ev.m = size.GetWidth()-wBorder;
//  ev.n = size.GetHeight()-hBorder;

//    Printf("Resize %d %d %d %d\n",ev.i,ev.j,ev.m,ev.n);
    ProcessNextEvent(&ev);
  }

  else {    
	ev.type = Draw;

    wxRegionIterator upd(GetUpdateRegion()); // get the update rect list

    while (upd) {
		ev.i = upd.GetX();
		ev.j = upd.GetY();
        ev.m = upd.GetW();
        ev.n = upd.GetH();
//        Printf("Paint %d %d %d %d %d\n",i,ev.i,ev.j,ev.m,ev.n);
        ProcessNextEvent(&ev);
        upd ++ ;
        i++;
	}
  }
  
  dc = NULL;

}

Canvas::Canvas(Frame *f) : wxWindow(f, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                                            wxSUNKEN_BORDER )
/*wxScrolledWindow(f, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                            wxSUNKEN_BORDER ) */
{
  SetBackgroundColour(*wxWHITE);
  frame = f;

  dc = NULL;
  wxClientDC dc1(this);
  dc1.SetPen(*wxBLACK_PEN);
  dc1.SetBrush(*wxTRANSPARENT_BRUSH);
  
  mouseButton = NoButton;
  flagResizeEvent = false;
}


Frame::Frame(wxString title,int x, int y, int w,int h) : wxFrame(NULL, -1, title,wxPoint(x,y),wxSize(w,h),wxDEFAULT_FRAME_STYLE | wxWANTS_CHARS)
{
  // Create the canvas
  canvas = new Canvas(this);

  // Useful ??  
//  canvas->SetFocusIgnoringChildren();

  // Allows to resize the canvas
  SendSizeEvent();
  XXFlushEvents();

  // Compute frame border
  int w1,h1;
  GetClientSize(&w1,&h1);
  hBorder = h-h1;
  wBorder = w-w1;

  // Compute canvas border
  canvas->GetClientSize(&w1,&h1);
  int w2,h2;
  canvas->GetSize(&w2,&h2);
  canvas->hBorder = h2-h1;
  canvas->wBorder = w2-w1;
  
  SetSize(-1,-1,w+totalWBorder(),h+totalHBorder());

//  SetWindowStyle(GetWindowStyleFlag() | wxNO_FULL_REPAINT_ON_RESIZE);
}

Frame::~Frame() 
{
}




void XXOpenGraphics(void)
{
  thePen = *wxBLACK_PEN; 
  theBrush = *wxBLACK_BRUSH;
  theFont =  wxFont(12,wxFONTFAMILY_DEFAULT,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL);
  thePenMode = wxCOPY;

/*  stupFrame *f = new stupFrame;
  f->Show(true);
  f->Raise();
  
    wxClientDC dc(f);  */
/*    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);*/
  //  dc.DrawLine(0,0,100,100);

}

void XXCloseGraphics(void)
{
}


/* Set the style of the line */
void XXSetLineStyle(FRAME frame,int flag)
{
  if (flag == LinePlain) thePen.SetStyle(wxSOLID);
  else thePen.SetStyle(wxSHORT_DASH);
  if (((Frame *) frame)->canvas->dc != NULL) ((Frame *) frame)->canvas->dc->SetPen(thePen);
}

/* Change the size of the pen of the Graphic Port */
void XXSetPenSize(FRAME frame,int size)
{ 
  thePen.SetWidth(size);
  if (((Frame *) frame)->canvas->dc != NULL) ((Frame *) frame)->canvas->dc->SetPen(thePen);
}

/* Set the pen mode of the Graphic Port */
void XXSetPenMode(FRAME frame,int mode)
{ 
  if (mode == PenPlain) thePenMode = wxCOPY;
  else thePenMode = wxINVERT;
  
  if (((Frame *) frame)->canvas->dc != NULL) ((Frame *) frame)->canvas->dc->SetLogicalFunction(thePenMode);
}


/* Draw a line in the graphic port */
void XXDrawLine(FRAME frame,int x,int y,int x1,int y1)
{ 
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);
    dc.DrawLine(x, y, x1, y1);
  }
  else {
    ((Frame *) frame)->canvas->dc->DrawLine(x, y, x1, y1);
  }
}

/* Draw a point in the Graphic Port */
void XXDrawPoint(FRAME frame,int x,int y)
{
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);
    dc.DrawPoint(x, y);
  }
  else {
    ((Frame *) frame)->canvas->dc->DrawPoint(x, y);
  }
  
}

/* Draw an ellipse in a rect */
void XXDrawEllipse(FRAME frame,int x,int y,int dx, int dy)
{
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);  
    dc.DrawEllipse(x, y, dx, dy);
  }
  else {
    ((Frame *) frame)->canvas->dc->SetBrush(*wxTRANSPARENT_BRUSH);
    ((Frame *) frame)->canvas->dc->DrawEllipse(x, y, dx, dy);
  }
}

/* Fill an ellipse in a rect */
void XXFillEllipse(FRAME frame,int x,int y,int dx, int dy)
{
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(theBrush);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);
    dc.DrawEllipse(x, y, dx, dy);
  }
  else {
    ((Frame *) frame)->canvas->dc->SetPen(thePen);
    ((Frame *) frame)->canvas->dc->SetBrush(theBrush);
    ((Frame *) frame)->canvas->dc->DrawEllipse(x, y, dx, dy);
  }
}


/* Draw a rectangle */
void XXDrawRect(FRAME frame,int x,int y,int dx,int dy)
{
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);  
    dc.DrawRectangle(x, y, dx, dy);
  }
  else {
    ((Frame *) frame)->canvas->dc->SetBrush(*wxTRANSPARENT_BRUSH);
    ((Frame *) frame)->canvas->dc->DrawRectangle(x, y, dx, dy);
  }

}

/* Fill a rectangle */
void XXFillRect(FRAME frame,int x,int y,int dx,int dy)
{
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(theBrush);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetLogicalFunction(thePenMode);
    dc.DrawRectangle(x, y, dx, dy);
  }
  else {
    ((Frame *) frame)->canvas->dc->SetPen(thePen);
    ((Frame *) frame)->canvas->dc->SetBrush(theBrush);
    ((Frame *) frame)->canvas->dc->DrawRectangle(x, y, dx, dy);
  }
} 

/* Set the clip rect */
void XXSetClipRect(FRAME frame, int x, int y, int w, int h)
{
  theClipX = x;
  theClipY = y;
  theClipW = w;
  theClipH = h;

  if (((Frame *) frame)->canvas->dc != NULL) {
    ((Frame *) frame)->canvas->dc->DestroyClippingRegion();
    ((Frame *) frame)->canvas->dc->SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
  }
}


/*
 * Managing Pixmaps
 */

static wxImage *im = NULL;
void XXAllocPixMap(int w,int h,unsigned char **pData,int *pRowBytes)
{
  im = new wxImage(w,h,false);
  *pData = (im->GetData());
  *pRowBytes = w*3;
}

void XXDeletePixMap(void)
{  
  delete im;
  im = NULL;
}
  
/* Display an image in a frame */
void XXDisplayPixMap(FRAME frame,int winX,int winY)
{
  wxBitmap b(*im);
  
  // Create a memory DC
  wxMemoryDC temp_dc;
  temp_dc.SelectObject(b);

  // We can now draw into the memory DC...
  // Copy from this DC to another DC.
  wxClientDC dc1(((Frame *) frame)->canvas);
  dc1.Blit(winX,winY, b.GetWidth(), b.GetHeight(),& temp_dc, 0, 0);

}

char XXIsDisplayBLittle(void)
{
  return(NO);
}

void XXFlush(void)
{
}

/********************************************************/
/*                Other functions                       */
/********************************************************/

/* Delete a window */
void XXDeleteFrame(FRAME frame)
{
  ((Frame *) frame)->Destroy();
}

/* Create a window */
FRAME XXNewFrame(char *title,int x, int y, int w,int h)
{
  wxWindow *ww = wxWindow::FindFocus();

  AHASHELEM e;
  WINDOW win;
  win = NULL;
  for (unsigned r = 0; r<theWindowsHT->nRows;r++) {
    for (e = (AHASHELEM) theWindowsHT->rows[r]; e != NULL; e = (AHASHELEM) e->next) {    
      win = (WINDOW) e;
	  if (win->frame != NULL && (((Frame *) (win->frame))->canvas == ww || ((Frame *) (win->frame)) == ww)) break;
	  win = NULL;
    }
  }
  if (win == NULL) ww = myApp->terminal;
  else ww = ((Frame *) (win->frame));

  Frame * f = new Frame(ANSI2WXSTRING(title),x,y,w,h);
  f->Show(true);

  ww->SetFocus();
  ww->Raise();
  return(f);
}

/* Change the size the position and the title of a window */
void XXChangeFrame(FRAME frame,char *title,int x,int y,int w, int h)
{
  int x1,y1,w1,h1;

  ((Frame *) frame)->canvas->GetClientSize(&w1,&h1);
  ((Frame *) frame)->GetPosition(&x1,&y1);

  if (w != w1 || h != h1) {
    ((Frame *) frame)->SetSize(x,y,w+((Frame *) frame)->totalWBorder(),h+((Frame *) frame)->totalHBorder());
  }
  else if (x != x1 || y != y1) {
    ((Frame *) frame)->Move(x,y);
  }
   
  if (title) ((Frame *) frame)->SetTitle(ANSI2WXSTRING(title));
}

FRAME XXGetFrontFrame(void)
{
 return(NULL);
}

void XXFrontFrame(FRAME frame)
{
  ((Frame *) frame)->Raise();
  ((Frame *) frame)->canvas->SetFocus();
}

void XXErrorFrame(FRAME frame)
{
  ((Frame *) frame)->canvas->dc = NULL;
  ((Frame *) frame)->canvas->flagResizeEvent = false;
  ((Frame *) frame)->canvas->mouseButton = NoButton;
  if (((Frame *) frame)->canvas->HasCapture()) ((Frame *) frame)->canvas->ReleaseMouse();
}


/********************************************************/
/*               Event managing                         */
/* Return the next event of type EventMask on the queue */
/* Return 0 if no event                                 */
/* EventMask =     
     1- BUTTONPRESS - Param : x,y,button
     2- REFRESH     - Param : x,y,dx,dy
        The parameters corresponds to the area of the
	window which has to be refreshed.
        This event must take care of the change
	of size/position of the window (this is indicated
	by dx < 0)
     3- KEYPRESS/KEYRELEASE    - Param : key,x,y
     4- ENTERWINDOW - no param
     5- LEAVEWINDOW - no param
   Any combination of those is possible                 */
/********************************************************/

/* Turn the autorepeat on */
void XXAutoRepeatOn(void)
{
}

/* Turn the autorepeat off */
void XXAutoRepeatOff(void)
{
}


/* Change the color of the Graphic port */
void XXSetColor(FRAME frame, unsigned long pixel)
{
  unsigned char r,g,b;
  
  thePalette->GetRGB(pixel,&r,&g,&b);
  theR = r;
  theG = g;
  theB = b;

  thePen.SetColour(r,g,b);
  theBrush.SetColour(r,g,b);
}


/*
 * Set the current colormap according to the values of red, green and blue. 
 * Specifies the index of the color of the cursor and of the colors that must
 * behave 'properly' during a PenInverse mode.
 * Returns the number of color cells that are used.
 */

int XXSetColormap(unsigned short red[],unsigned short green[],unsigned short blue[],
		  unsigned long pixels[],int nCols,int flagSharedColormap, int mouseMode,
		  unsigned short mouseRed, unsigned short mouseGreen, unsigned short mouseBlue)
{
  int i;
  unsigned char cred[32000];
  unsigned char cgreen[32000];
  unsigned char cblue[32000];
  
  if (thePalette != NULL) {
    delete thePalette;
    thePalette = NULL;
  }

  for (i=0;i<nCols;i++) {
    cred[i] = (unsigned char) (red[i]/256);
    cblue[i] = (unsigned char) (blue[i]/256);
    cgreen[i] = (unsigned char) (green[i]/256);
  }
  
  thePalette = new wxPalette(nCols,cred,cgreen,cblue);
  
  for (i=0;i<nCols;i++) {
    pixels[i] = thePalette->GetPixel(cred[i],cgreen[i],cblue[i]);
  }
  
  return(nCols);
}


/*
 * Animate one color of the colormap
 */

void XXAnimateColor(unsigned long pixel, unsigned short r, unsigned short g, unsigned short b)
{
}


/*
 * Returns the number of colors available 
 */
int XXNumOfColors(void)
{
return(65000);
}

/*
 * Returns the depth
 */
int XXDepth(void)
{
return(16);
}

/*
 * Is the screen BW ?
 */

int XXIsBWScreen(void)
{
return(NO);
}

/*
 * Type of the screen
 */

char * XXScreenType(void)
{
  return("wx");
}

void XXScreenRect(int *x, int *y, int *w, int *h)
{
  *x = 0;
  *y = 0;
  *w = wxSystemSettings::GetMetric(wxSYS_SCREEN_X);
  *h = wxSystemSettings::GetMetric(wxSYS_SCREEN_Y);
}

/*
 * Get the foreground and background colors of the terminal window 
 */

void XXBgColor(unsigned short *r,unsigned short *g,unsigned short *b)
{
 wxColour col =  wxSystemSettings::GetColour(wxSYS_COLOUR_APPWORKSPACE);
 
 *r = ((int) col.Red())*256;
 *g = ((int) col.Green())*256;
 *b = ((int) col.Blue())*256;
}

void XXFgColor(unsigned short *r,unsigned short *g,unsigned short *b)
{
 wxColour col =  wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
 
 *r = ((int) col.Red())*256;
 *g = ((int) col.Green())*256;
 *b = ((int) col.Blue())*256;
}




/* Draw a string in the graphic port */
void XXDrawString(FRAME frame,int x,int y,char *str, double angle)
{
  wxString s = ANSI2WXSTRING(str);
  if (((Frame *) frame)->canvas->dc == NULL) {
    wxClientDC dc(((Frame *) frame)->canvas);  
    dc.SetPen(thePen);
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    dc.DestroyClippingRegion();
    dc.SetClippingRegion(theClipX,theClipY,theClipW,theClipH);
    dc.SetFont(theFont);
    int w,h,d,i;
    dc.GetTextExtent(s,&w,&h,&d,&i);
    if (angle == 0) dc.DrawText(s,x, y-(h-d));
    else dc.DrawRotatedText(s,x, y-(h-d),angle);
  }
  else {
    int w,h,d,i;
    ((Frame *) frame)->canvas->dc->GetTextExtent(s,&w,&h,&d,&i);
    if (angle == 0) ((Frame *) frame)->canvas->dc->DrawText(s,x, y-(h-d));
    else ((Frame *) frame)->canvas->dc->DrawRotatedText(s,x, y-(h-d),angle);
  }

}


char *XXGetDefaultFont(FONTID *fontStruct,int *size) 
{
  *size = 10;
  return("default");
}




void XXGetFontInfo(FONT font, int *ascent, int *descent, int *interline)
{
  wxScreenDC dc;
    
  wxString s(_T("gfy|pA\193\198"));
  wxFontFamily ff;
  char *face = NULL;
  
  if (!strcmp(font->fontName,"default")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"decorative")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"roman")) ff = wxFONTFAMILY_ROMAN;
  else if (!strcmp(font->fontName,"script")) ff = wxFONTFAMILY_SCRIPT;
  else if (!strcmp(font->fontName,"swiss")) ff = wxFONTFAMILY_SWISS;
  else if (!strcmp(font->fontName,"modern")) ff = wxFONTFAMILY_MODERN;
  else if (!strcmp(font->fontName,"teletype")) ff = wxFONTFAMILY_TELETYPE;
  else {
    ff = wxFONTFAMILY_DEFAULT;
    face = font->fontName;
  }
  if (face == NULL) face = "";
  
  int style,weight;
  
  if (font->style == FontPlain) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontItalic) {
    style = wxFONTSTYLE_ITALIC;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontBold) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_BOLD;
  }
  
  wxFontEncoding fe;
  if (font->encoding = FontEncodingDefault) fe = wxFONTENCODING_DEFAULT;
  else if (font->encoding = FontEncodingGreek) fe = greekEncoding;
  
  wxFont f(font->size, ff, style, weight,false,ANSI2WXSTRING(face),fe);
  wxFont f1 = dc.GetFont();
  dc.SetFont(f);
  
  int w,h,d,i;  
  dc.GetTextExtent(s,&w,&h,&d,&i);
  
  *ascent = h-d;
  *descent = d;
  *interline = 1;
  
/*  *ascent = 1;
  *descent = 0;
  */
  dc.SetFont(f1);
}

int XXGetStringWidth(FONT font, char *text)
{
  wxScreenDC dc;
    
  wxString s = ANSI2WXSTRING(text);
  wxFontFamily ff;
  char *face = NULL;
  
  if (!strcmp(font->fontName,"default")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"decorative")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"roman")) ff = wxFONTFAMILY_ROMAN;
  else if (!strcmp(font->fontName,"script")) ff = wxFONTFAMILY_SCRIPT;
  else if (!strcmp(font->fontName,"swiss")) ff = wxFONTFAMILY_SWISS;
  else if (!strcmp(font->fontName,"modern")) ff = wxFONTFAMILY_MODERN;
  else if (!strcmp(font->fontName,"teletype")) ff = wxFONTFAMILY_TELETYPE;
  else {
    ff = wxFONTFAMILY_DEFAULT;
    face = font->fontName;
  }
  if (face == NULL) face = "";
  
  int style,weight;
  
  if (font->style == FontPlain) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontItalic) {
    style = wxFONTSTYLE_ITALIC;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontBold) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_BOLD;
  }

  wxFontEncoding fe;
  if (font->encoding = FontEncodingDefault) fe = wxFONTENCODING_DEFAULT;
  else if (font->encoding = FontEncodingGreek) fe = greekEncoding;
  
  wxFont f(font->size, ff, style, weight,false,ANSI2WXSTRING(face),fe);
  wxFont f1 = dc.GetFont();
  dc.SetFont(f);
  
  int w,h,d,i;  
  dc.GetTextExtent(s,&w,&h,&d,&i);
  
  return(w);
}

int XXGetStringHeight(FONT font, char *text)
{
  wxScreenDC dc;
  
  wxString s=ANSI2WXSTRING(text);
  wxFontFamily ff;
  char *face = NULL;
  
  if (!strcmp(font->fontName,"default")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"decorative")) ff = wxFONTFAMILY_DEFAULT;
  else if (!strcmp(font->fontName,"roman")) ff = wxFONTFAMILY_ROMAN;
  else if (!strcmp(font->fontName,"script")) ff = wxFONTFAMILY_SCRIPT;
  else if (!strcmp(font->fontName,"swiss")) ff = wxFONTFAMILY_SWISS;
  else if (!strcmp(font->fontName,"modern")) ff = wxFONTFAMILY_MODERN;
  else if (!strcmp(font->fontName,"teletype")) ff = wxFONTFAMILY_TELETYPE;
  else {
    ff = wxFONTFAMILY_DEFAULT;
    face = font->fontName;
  }
  if (face == NULL) face = "";
  
  int style,weight;
  
  if (font->style == FontPlain) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontItalic) {
    style = wxFONTSTYLE_ITALIC;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontBold) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_BOLD;
  }
  
  wxFontEncoding fe;
  if (font->encoding = FontEncodingDefault) fe = wxFONTENCODING_DEFAULT;
  else if (font->encoding = FontEncodingGreek) fe = greekEncoding;
  
  wxFont f(font->size, ff, style, weight,false,ANSI2WXSTRING(face),fe);
  wxFont f1 = dc.GetFont();
  dc.SetFont(f);
  
  int w,h,d,i;  
  dc.GetTextExtent(s,&w,&h,&d,&i);
  
  return(h);
}

// If frame is NULL then set font to the terminal
void XXSetFont(FRAME frame,FONT font)
{
  wxFontFamily ff;
  char *face = NULL;

  if (!strcmp(font->fontName,"default")) {
    ff = wxFONTFAMILY_DEFAULT;
    font->encoding = FontEncodingDefault;
  }
  else if (!strcmp(font->fontName,"decorative")) ff = wxFONTFAMILY_DECORATIVE;
  else if (!strcmp(font->fontName,"roman")) ff = wxFONTFAMILY_ROMAN;
  else if (!strcmp(font->fontName,"script")) ff = wxFONTFAMILY_SCRIPT;
  else if (!strcmp(font->fontName,"swiss")) ff = wxFONTFAMILY_SWISS;
  else if (!strcmp(font->fontName,"modern")) ff = wxFONTFAMILY_MODERN;
  else if (!strcmp(font->fontName,"teletype")) ff = wxFONTFAMILY_TELETYPE;
  else {
    ff = wxFONTFAMILY_DEFAULT;
    face = font->fontName;
  }
  if (face == NULL) face = "";
     
  //  ff = wxFONTFAMILY_TELETYPE;

  int style,weight;
  
  if (font->style == FontPlain) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontItalic) {
    style = wxFONTSTYLE_ITALIC;
    weight = wxFONTWEIGHT_NORMAL;
  }
  else if (font->style == FontBold) {
    style = wxFONTSTYLE_NORMAL;
    weight = wxFONTWEIGHT_BOLD;
  }

  wxFontEncoding fe;
  if (font->encoding == FontEncodingDefault) fe = wxFONTENCODING_DEFAULT;
  else if (font->encoding == FontEncodingGreek) {fe = greekEncoding;}
  
  wxFont f(font->size, ff, style, weight,false,ANSI2WXSTRING(face),fe);
  
  theFont = f;
  
  if (frame != NULL) {
    if (((Frame *) frame)->canvas->dc != NULL) ((Frame *) frame)->canvas->dc->SetFont(theFont);
  }
  else {
    myApp->terminal->textCtrl->SetFont(theFont);
  }
}


// ????
void XXFontMatch(char *name, int encoding, char flagFixed, LISTV lv)
{
/*  static char *list[] = {"default","decorative","roman","script","swiss","modern","teletype",NULL};
  char **l = list;
  
  while (*l != NULL) {
    if (MatchStr(*l,name))  Append2ListvStr(lv,*l);
    l++;
  }
*/
  
  wxFontEnumerator fontEnumerator;
  
  switch (encoding) {
    case FontEncodingGreek : 
      fontEnumerator.EnumerateFacenames(greekEncoding);
      break;
    case FontEncodingDefault :
      fontEnumerator.EnumerateFacenames(wxFONTENCODING_DEFAULT,(flagFixed!=0 ? true : false));
      break;
    default : 
      Errorf("XXFontMatch() : Unknown font encoding %d",encoding);
  }
   
  // This is a change from wxWidget version 2.6 to 2.8
#ifndef __WX26__
  int nFacenames = fontEnumerator.GetFacenames().Count();    
#else
  int nFacenames = fontEnumerator.GetFacenames()->Count();    
#endif
  
  wxString facename;
  for (int i = 0; i< nFacenames; i++) {    
#ifndef __WX26__
    facename = fontEnumerator.GetFacenames().Item(i);
#else
    facename = fontEnumerator.GetFacenames()->Item(i);
#endif
    if (MatchStr(WXSTRING2ANSI(facename),name)) AppendStr2Listv(lv,WXSTRING2ANSI(facename));
  }
}


// ????
char XXExistFont(char *name,int size, unsigned char style, unsigned char encoding, FONTID *fontStruct)
{
  return(YES);
}


extern "C" {

void C_System(char **argv)
{
  char *str;
  char **l;
  long *bb,c;
  int n;

  argv = ParseArgv(argv,tWORD,&str,-1);
  
  if (!strcmp(str,"nEvents")) {
    NoMoreArgs(argv);
    SetResultInt(toplevelCur->nEvents+1);
  }
  else if (!strcmp(str,"mouse")) {
    argv = ParseArgv(argv,tINT,&n,-1);
    switch (n) {
      case 1 : nbOfButtons = 1; break;
      case 2 : nbOfButtons = 2; break;
      case 3 : nbOfButtons = 3; break;
      default: Errorf("The number of buttons should be 1 or 2 or 3");
    }
    for (int i = 0;i< 3-nbOfButtons;i++) {
      argv = ParseArgv(argv,tLIST,&l,-1);
      if (l[0] == NULL) ErrorUsage();
      if (!strcmp(l[0],"left")) bb = &leftButton;
      else if (!strcmp(l[0],"right")) bb = &rightButton;
      else if (!strcmp(l[0],"middle")) bb = &middleButton;
      else Errorf("Bad mouse button name '%s'",l[0]);
      int j = 1;
      if (l[j] == NULL) ErrorUsage();
      if (!strcmp(l[j],"opt")) {c = ModOpt; j=2;}
      else if (!strcmp(l[1],"meta")) {c = ModMeta; j=2;}
      else c = 0;
      if (l[j] == NULL) ErrorUsage();
      if (l[j+1] != NULL) ErrorUsage();
      if (!strcmp(l[j],"right")) c += RightButton;
      else if (!strcmp(l[j],"middle")) c += MiddleButton;
      else if (!strcmp(l[j],"left")) c += LeftButton;
      else ErrorUsage();
      *bb = c;
    }
    NoMoreArgs(argv);
  }    
}

}


void XXFlushEvents()
{
  myApp->Yield(true);
}

